#!/usr/bin/env	python3


"""
Automated run of ComparingPartitions (May 2021) to identify regions of cluster stability (i.e. cg/wgMLST partition ranges in which cluster composition is similar)
original github repository: https://github.com/jacarrico/ComparingPartitions
By Veronica Mixao
@INSA
comparing_partitions_v2.py runs one of two options: 
A) comparing partitions of a single method to determine stability regions (where stability regions correspond to partition threshold ranges where 
at least 5 subsequent partitions have an Adjusted Wallace above 0.99) 
comparing_partitions_v2.py -i1 PARTITIONS -o1 0 -a stability -t OUTPUT_NAME -n 5 -thr 0.99 -log LOG
B) comparing partitions between two different methods to determine their congruence (STILL UNDER DEVELOPMENT... RUN AT YOUR OWN RISK)
comparing_partitions_v2.py -i1 PARTITIONS -i2 PARTITIONS2 -o1 0 -o2 0 -a between_methods -t OUTPUT_NAME -n 5 -thr 0.99 -log LOG
"""


import sys
import math
import pandas 
import numpy as np
import argparse
from metrics import getContTable, getContTableTotals, getMismatchMatrix, getAdjRand, getSimpsons, getWallace, getAdjustedWallace
import textwrap
import random


# functions	----------

def rm_redundant(matrix, log):
	""" determine the most discriminatory cut-off
	and remove redundant samples """
	
	# what is the most discriminatory column?
	
	max_size,main_col = 0,""
	for col in matrix.columns[1:]:
		size = len(set(matrix[col].values.tolist()))
		if size > max_size:
			max_size,main_col = size,col
	print("\tThe most discriminatory column found was " + main_col)
	print("\tThe most discriminatory column found was " + main_col, file = log)
	
	# get samples
	
	final_samples = []
	for cluster in set(matrix[main_col].values.tolist()):
		flt_data = matrix[matrix[main_col] == cluster]
		selected_sample = random.choice(flt_data[flt_data.columns[0]].values.tolist())
		final_samples.append(selected_sample)
	
	if len(final_samples) != max_size:
		print("\tSampling got more samples than clusters... something went wrong :-(")
		print("\tSampling got more samples than clusters... something went wrong :-(", file = log)
		sys.exit()
	
	# final matrix
	
	matrix = matrix[matrix[matrix.columns[0]].isin(final_samples)]
	
	return matrix	
			
	
def	order_columns(matrix):
	""" Order the columns """
	
	cols = matrix.columns.tolist()
	matrix_ordered = pandas.concat([matrix[cols[0]], matrix[reversed(cols[1:])]], axis=1, join="inner")
	
	return matrix_ordered
	

def	metrics4stability(matrix, tag, log):
	""" Comparing partitions of a single method """
	
	with open(tag + "_metrics.tsv", "w+") as out:
		
		#prepare output
		
		cols = ["col2", "col1", "AdjRand", "Simpson_col2", "Simpson_col1", "Wallace", "AdjWallace", "AdjWallaceIClow", "AdjWallaceIChigh", "col2_cluster", "col1_cluster"]
		print("\t".join(cols), file = out)
		
		# calculating metrics
		
		i = 0
			
		for col1 in matrix.columns:
			i += 1
			j = 0
			for col2 in matrix.columns:
				j += 1
				if col1 != col2:
					if j-i == 1:
						print("\t\t",col2," => ",col1)
						icol = matrix[col1].values.tolist()
						jcol = matrix[col2].values.tolist()
						cont = getContTable(icol, jcol)
						cont_2 = getContTable(jcol, icol)
						abcdn = getMismatchMatrix(cont, icol, jcol)
								
						sid1 = getSimpsons(icol)
						sid2 = getSimpsons(jcol)
						
						wallace = getWallace(abcdn[0], abcdn[1], abcdn[2])
						
						AdjustedWallace = getAdjustedWallace(cont_2, jcol, icol)
						print("\t\t\tAdjusted Wallace ",col2," vs ",col1,"\t%f\t%f\t%f" % AdjustedWallace)
						print("\t\t\tAdjusted Wallace ",col2," vs ",col1,"\t%f\t%f\t%f" % AdjustedWallace, file = log)
						
						print("\t".join([str(col2), str(col1), str(getAdjRand(icol,jcol)), str(sid2[0]), str(sid1[0]), str(wallace[0]), str(AdjustedWallace[0]), str(AdjustedWallace[1]), str(AdjustedWallace[2]), str(sid2[3]), str(sid1[3])]), file = out)


def	comparing_methods(matrix1, matrix2, tag, log):
	""" Comparing partitions of two methods """
	
	# defining dictionaries for df

	AdjWallace1 = {} #dictionary with all the Adjusted Wallace values from file1 to file2 (keys = file2)
	AdjWallace2 = {} #dictionary with all the Adjusted Wallace values from file2 to file1 (keys = file1)
	Wallace1 = {} #dictionary with all the Wallace values from file1 to file2 (keys = file2)
	Wallace2 = {} #dictionary with all the Wallace values from file2 to file1 (keys = file1)
	Simpsons1 = {} #dictionary with all the Simpsons values for file1 (keys = columns)
	Simpsons2 = {} #dictionary with all the Simpsons values for file2 (keys = columns)
	Rand = {} #dictionary with all the Rand metrics (keys = file2)
	final_metrics = {} #dictionary with the sum of Adjusted Wallace 1, Adjusted Wallace 2 and Adjusted Rand

	AdjWallace1["0_method_partition"] = []
	AdjWallace2["0_method_partition"] = []
	Wallace1["0_method_partition"] = []
	Wallace2["0_method_partition"] = []
	Rand["0_method_partition"] = [] 
	final_metrics["0_method_partition"] = [] 

	#determine all the columns for the dataframe
	
	for col1 in matrix1.columns:
		AdjWallace1["0_method_partition"].append(col1)
		Wallace1["0_method_partition"].append(col1)
		AdjWallace2["0_method_partition"].append(col1)
		Wallace2["0_method_partition"].append(col1)
		Rand["0_method_partition"].append(col1)
		final_metrics["0_method_partition"].append(col1)
		
	for col2 in matrix2.columns:
		AdjWallace1[col2] = []
		Wallace1[col2] = []
		Rand[col2] = []
		AdjWallace2[col2] = []
		Wallace2[col2] = []
		final_metrics[col2] = []

	#performing the comparisons
		
	for col1 in matrix1.columns:
		for col2 in matrix2.columns:
			print("\t\t",col1," ---------- ",col2)
			print("\t\t",col1," ---------- ",col2, file = log)
			icol = matrix1[col1].values.tolist()
			jcol = matrix2[col2].values.tolist()
			cont = getContTable(icol, jcol)
			cont_2 = getContTable(jcol, icol)
			abcdn = getMismatchMatrix(cont, icol, jcol)
			abcdn_2 = getMismatchMatrix(cont_2, jcol, icol)
				
			#getting rand metric
			rand_metric = getAdjRand(icol,jcol)
			Rand[col2].append(rand_metric)
				
			#getting simpson's metric
			sid1 = getSimpsons(icol)
			sid2 = getSimpsons(jcol)
				
			Simpsons1[col1] = sid1
			Simpsons2[col2] = sid2 
				
			#getting Wallace metric
			wallace = getWallace(abcdn[0], abcdn[1], abcdn[2])
				
			Wallace1[col2].append(wallace[0])
			Wallace2[col2].append(wallace[0])
						
			#getting Adjusted Wallace metric
			AdjustedWallace = getAdjustedWallace(cont, icol, jcol)
			AdjustedWallace_2 = getAdjustedWallace(cont_2, jcol, icol)
			
			AdjWallace1[col2].append(AdjustedWallace[0])
			AdjWallace2[col2].append(AdjustedWallace_2[0])
			
			#getting final metrics
			final_score = rand_metric + AdjustedWallace[0] + AdjustedWallace_2[0]
			
			final_metrics[col2].append(final_score)

	df_Rand = pandas.DataFrame(data = Rand)
	df_Simpsons1 = pandas.DataFrame(data = Simpsons1) #needs to be transposed
	df_Simpsons2 = pandas.DataFrame(data = Simpsons2) #needs to be transposed
	df_Wallace1 = pandas.DataFrame(data = Wallace1)
	df_Wallace2 = pandas.DataFrame(data = Wallace2)
	df_AdjWallace1 = pandas.DataFrame(data = AdjWallace1)
	df_AdjWallace2 = pandas.DataFrame(data = AdjWallace2)
	df_final_metrics = pandas.DataFrame(data = final_metrics)

	df_Simpsons1T = df_Simpsons1.T
	df_Simpsons2T = df_Simpsons2.T

	df_Rand.to_csv(tag + "_AdjustedRand.tsv", index = False, header= True, sep ="\t")
	df_Simpsons1T.to_csv(tag + "_Simpsons1.tsv", index = True, header= False, sep ="\t")
	df_Simpsons2T.to_csv(tag + "_Simpsons2.tsv", index = True, header= False, sep ="\t")
	df_Wallace1.to_csv(tag + "_Wallace1.tsv", index = False, header= True, sep ="\t")
	df_Wallace2.to_csv(tag + "_Wallace2.tsv", index = False, header= True, sep ="\t")
	df_AdjWallace1.to_csv(tag + "_AdjWallace1.tsv", index = False, header= True, sep ="\t")
	df_AdjWallace2.to_csv(tag + "_AdjWallace2.tsv", index = False, header= True, sep ="\t")
	df_final_metrics.to_csv(tag + "_final_score.tsv", index = False, header= True, sep ="\t")


def	get_stability(infile, thr, n, tag, log):
	""" Determine stability regions of a single method """
	
	print("\t\tDetermining stability regions with threshold: ", str(thr), " and minimum n of ", str(n))
	print("\t\tDetermining stability regions with threshold: ", str(thr), " and minimum n of ", str(n), file = log)
	
	with open(infile, "r") as f_open:
		with open(tag + "_stableRegions.tsv", "w+") as out:
			f = f_open.readlines()
			i = 0
			j = 0
			counter = 0
			stable_blocks = {}
			
			for line in f:
				i += 1
				if i > 1:
					l = line.split("\t")
					score = float(l[6])
					
					if score >= float(thr):
						info = l[0] + "->" + l[1]
						if j == 0:
							j += 1
							counter += 1
							stable_blocks["block_" + str(counter)] = []
						stable_blocks["block_" + str(counter)].append(info)
					else:
						j = 0
			
			print("#Stability regions of the Adjusted Wallace coefficient for ", infile, "\n#using a threshold of ", str(thr), " and minimum number of required observations of ", str(n), file = out)
			print("#block_id\tfirst_partition\tlast_partition\tlen_block", file = out)
			
			for block in stable_blocks.keys():
				if len(stable_blocks[block]) >= int(n):
					print("\t".join([block, stable_blocks[block][0], stable_blocks[block][-1], str(len(stable_blocks[block]))]), file = out)
					print("\t\t", block, stable_blocks[block][0], stable_blocks[block][-1], str(len(stable_blocks[block])))
					print("\t\t", block, stable_blocks[block][0], stable_blocks[block][-1], str(len(stable_blocks[block])), file = log)


# pipeline ----------
	
def main():
	
	# define options
	parser = argparse.ArgumentParser(prog="comparing_partitions_v2.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                         comparing_partitions_v2.py                          #
									#                                                                             #
									###############################################################################  
									                            
									comparing_partitions_v2 identifies regions of cluster stability (i.e. cg/wgMLST 
									partition threshold ranges in which cluster composition is similar)
									
									
									This is a modified version of:
									https://github.com/jacarrico/ComparingPartitions
									
									
									By default, for the stability analysis, this script searches for the most 
									discriminatory partition and only keeps one random samples per cluster. This 
									option can be reversed with the option '--keep-redundants'.
									comparing_partitions_v2.py runs one of two options: 
									
									A) comparing partitions of a single method to determine stability regions (where 
									stability regions correspond to partition threshold ranges where at least 5 
									subsequent partitions have an Adjusted Wallace above 0.99) 
									comparing_partitions_v2.py -i1 PARTITIONS -o1 0 -a stability -t OUTPUT_NAME -n 5 
									-thr 0.99 -log LOG
									B) comparing partitions between two different methods to determine their congruence
									comparing_partitions_v2.py -i1 PARTITIONS -i2 PARTITIONS2 -o1 0 -o2 0 -a 
									between_methods -t OUTPUT_NAME -n 5 -thr 0.99 -log LOG
									
									WARNING!! Cluster congruence analysis between different methods is still under 
									development in the frame of the One Health EJP BeOne project for a major 
									comparison of surveillance pipelines! So, for now, use it at your own risk!
									
									-------------------------------------------------------------------------------"""))
	
	parser.add_argument("-i1", "--input1", dest="input1", action= "store", required=True, help="[MANDATORY] Input matrix 1 (table with partitions)")
	parser.add_argument("-i2", "--input2", dest="input2", action= "store", help="Input matrix 2 (table with partitions)")
	parser.add_argument("-o1", "--order1", dest="order1", action= "store", default=0, required=True, help="Partitions order in matrix 1 (0: min -> max; 1: max -> min) [0]")
	parser.add_argument("-o2", "--order2", dest="order2", action= "store", help="Partitions order in matrix 2 (0: min -> max; 1: max -> min) [0]")
	parser.add_argument("-a", "--analysis", dest="analysis", action= "store", default="between_methods", help="Type of analysis (options: stability or between_methods) [between_methods]")
	parser.add_argument("-t", "--tag", dest="tag", action= "store", required=True, help="[MANDATORY] Tag for output file name")
	parser.add_argument("-n", "--n_obs", dest="n_obs", action="store", default=5, help="Minimum number of sequencial observations to consider an interval for method stability analysis [5]")
	parser.add_argument("-thr", "--threshold", dest="threshold", action= "store", default=0.99, help="Threshold of Adjusted Wallace score to consider an observation for method stability \
						analysis [0.99]")
	parser.add_argument("--keep-redundants", dest="keep_redundants", action= "store_true", help="Set ONLY if you want to keep all samples of each cluster of the most discriminatory partition\
						 (by default redundant samples are removed to avoid the influence of cluster size)")
	parser.add_argument("-log", "--log", dest="log", action= "store", default="log", help="Log file")
						
	args = parser.parse_args()
	
	
	# starting logs	----------

	log_name = args.log
	log = open(log_name, "a+")
	
	print("\n-------------------- comparing_partitions_v2.py --------------------\n")
	print("\n-------------------- comparing_partitions_v2.py --------------------\n", file = log)
	print(" ".join(sys.argv))
	print(" ".join(sys.argv), file = log)
	
	
	# pipeline comparing methods 	----------
	
	if args.analysis == "between_methods":
		if not args.input2 or not args.order2:
			print("Provided options are incompatible! 'between_method' analysis requires input1, input2, order1 and order2!")
			print("Provided options are incompatible! 'between_method' analysis requires input1, input2, order1 and order2!", file = log)
			sys.exit()
		else:
			matrix1 = pandas.read_table(args.input1)
			matrix2 = pandas.read_table(args.input2)
		print("Preparing for comparing methods...")
		print("Preparing for comparing methods...", file = log)
		
		# remove redundant samples
		
		if not args.keep_redundants: # needs to remove redundant samples 
			print("Removing redundant samples...")
			print("Removing redundant samples...", file = log)
			matrix1 = rm_redundant(matrix1, log)
			matrix2 = rm_redundant(matrix2, log)
		
		
		# ordering
		
		print("Ordering matrix...")
		print("Ordering matrix...", file = log)
		
		if int(args.order1) != 0:
			if int(args.order1)  == 1:
				ordered_matrix1 = order_columns(matrix1)
			else:
				print("\tInvalid order of matrix1!")
				print("\tInvalid order of matrix1!", file = log)
				sys.exit()
		else:
			ordered_matrix1 = matrix1
		
		if int(args.order2) != 0:
			if int(args.order2) == 1:
				ordered_matrix2 = order_columns(matrix2)
			else:
				print("\tInvalid order of matrix1!")
				print("\tInvalid order of matrix1!", file = log)
				sys.exit()
		else:
			ordered_matrix2 = matrix2
		
		# filter matrix
		
		print("Filtering matrix...")
		print("Filtering matrix...", file = log)
		
		filtered_matrix1_not_ordered = ordered_matrix1.loc[ordered_matrix1[ordered_matrix1.columns[0]].isin(ordered_matrix2[ordered_matrix2.columns[0]])]
		filtered_matrix2_not_ordered = ordered_matrix2.loc[ordered_matrix2[ordered_matrix2.columns[0]].isin(ordered_matrix1[ordered_matrix1.columns[0]])]
		
		filtered_matrix1 = filtered_matrix1_not_ordered.sort_values(by=filtered_matrix1_not_ordered.columns[0])
		filtered_matrix2 = filtered_matrix2_not_ordered.sort_values(by=filtered_matrix2_not_ordered.columns[0])
		
		# cleaning
		
		print("Cleaning matrix...")
		print("Cleaning matrix...", file = log)
		
		clean_matrix1 = filtered_matrix1.drop(filtered_matrix1.columns[0], axis=1)
		clean_matrix2 = filtered_matrix2.drop(filtered_matrix2.columns[0], axis=1)

		# removing columns in which all samples form a cluster 

		for col in clean_matrix1.columns:
			if len(pandas.unique(clean_matrix1[col])) == 1:
				clean_matrix1 = clean_matrix1.drop(col, axis=1)
		for col in clean_matrix2.columns:
			if len(pandas.unique(clean_matrix2[col])) == 1:
				clean_matrix2 = clean_matrix2.drop(col, axis=1)

		print("\tN partitions matrix 1:", len(clean_matrix1.columns))
		print("\tN partitions matrix 2:", len(clean_matrix2.columns))

		# comparing methods
		
		print("Comparing methods...")
		print("Comparing methods...", file = log)
		
		comparing_methods(clean_matrix1, clean_matrix2, args.tag, log)
		
			
	# pipeline stability
	
	elif args.analysis == "stability":
		if args.input2 or args.order2:
			print("Provided options are incompatible! 'stability' analysis only accepts input1 and order1!")
			print("Provided options are incompatible! 'stability' analysis only accepts input1 and order1!", file = log)
			sys.exit()
		else:
			matrix = pandas.read_table(args.input1)
		print("Preparing for stability analysis...")
		print("Preparing for stability analysis...", file = log)
		
		# remove redundant samples
		
		if not args.keep_redundants: # needs to remove redundant samples 
			print("Removing redundant samples...")
			print("Removing redundant samples...", file = log)
			matrix = rm_redundant(matrix, log)
			
		# ordering
		
		print("Ordering matrix...")
		print("Ordering matrix...", file = log)
		
		if int(args.order1) != 0:
			if int(args.order1)  == 1:
				ordered_matrix = order_columns(matrix)
			else:
				print("\tInvalid order!")
				print("\tInvalid order!", file = log)
				sys.exit()
		else:
			ordered_matrix = matrix
		
		# cleaning
		
		print("Cleaning matrix...")
		print("Cleaning matrix...", file = log)
		
		clean_matrix = ordered_matrix.drop(ordered_matrix.columns[0], axis=1)
			
		# calculating metrics
		
		print("Calculating metrics...")
		print("Calculating metrics...", file = log)
		
		metrics4stability(clean_matrix, args.tag, log)
		
		# get stability
		
		print("Get stability...")
		print("Get stability...", file = log)
		
		get_stability(args.tag + "_metrics.tsv", args.threshold, args.n_obs, args.tag, log)
	
	
	# other options
	
	else:
		print("Invalid analysis type!")
		print("Invalid analysis type!", file = log)
		sys.exit()
	
	
	print("\ncomparing_partitions_v2.py is done!")
	print("\ncomparing_partitions_v2.py is done!", file = log)
	
	log.close()

# running pipeline
	
if __name__ == '__main__':
	main()



#!/usr/bin/env  python3

"""
This script can be used to generate the heatmap of the congruence score obtained 
for the comparison of two pipelines at all threshold levels, as well as 
determine the tendency of the respective corresponding points.
It takes as input the ALL_CORRESPONDENCE.tsv file (obtained with 
get_best_part_correspondence.py) and the *final_score.tsv file obtained with 
comparing_partitions_v2.py. 

By Veronica Mixao
@INSA
"""

import sys
import argparse
import textwrap
import pandas
import seaborn as sn
import matplotlib.pyplot as plt
from scipy import stats

version = "1.0.0"
last_updated = "2024-07-26"

# functions	----------

def get_tendency(correspondence, pipeline1, pipeline2, threshold):
	""" This function generates the heatmap 
	input: filename
	output: seaborn heatmap
	"""

	# TENDENCY

	possible_comparison_names = [pipeline1 + "_vs_" + pipeline2, pipeline2 + "_vs_" + pipeline1]

	mx = pandas.read_table(correspondence)
	all_comparisons = pandas.unique(mx[mx.columns[0]])
	
	comparison = ""
	for comp1 in all_comparisons:
		if "_rev" not in comp1:
			if comp1 in possible_comparison_names:
				if threshold != "":
					extension = comp1 + "_thr" + str(threshold) + "_slope.tsv"
				else:
					extension = comp1 + "_slope.tsv"
				with open(extension, "w+") as out:
					print("#comp1\tcomp2\tslope\tintercept\tr_value\tp_value\tstd_err", file = out)
					for comp2 in all_comparisons:
						if "_rev" in comp2:
							if comp2.split("_rev")[0] == comp1:
								comps = [comp1, comp2]
								flt_mx = mx.loc[mx[mx.columns[0]].isin(comps)]
								if threshold != "":
									flt_mx = flt_mx[(flt_mx["method1"] <= int(threshold)) & (flt_mx["method2"] <= int(threshold))]
								#sn.lmplot(x="method1", y="method2", hue=mx.columns[0], data=flt_mx)
								#plt.savefig(comp1 + ".png") 
								if len(flt_mx[flt_mx.columns[0]].values.tolist()) == 0:
									print("No trend line will be provided as no congruence point was found!!")
								else:
									if len(flt_mx[flt_mx[flt_mx.columns[0]] == comp1]["method1"].values.tolist()) != 0:
										slope, intercept, r_value, p_value, std_err = stats.linregress(flt_mx[flt_mx[flt_mx.columns[0]] == comp1]["method1"],flt_mx[flt_mx[flt_mx.columns[0]] == comp1]["method2"])
										print(comp1 + "\t" + comp2 + "\t" + str(slope) + "\t" + str(intercept) + "\t" + str(r_value) + "\t" + str(p_value) + "\t" + str(std_err), file = out)
									if len(flt_mx[flt_mx[flt_mx.columns[0]] == comp2]["method1"].values.tolist()) != 0:
										slope2, intercept2, r_value2, p_value2, std_err2 = stats.linregress(flt_mx[flt_mx[flt_mx.columns[0]] == comp2]["method1"],flt_mx[flt_mx[flt_mx.columns[0]] == comp2]["method2"])
										print(comp2 + "\t" + comp1 + "\t" + str(slope2) + "\t" + str(intercept2) + "\t" + str(r_value2) + "\t" + str(p_value2) + "\t" + str(std_err2), file = out)
									sn.lmplot(x="method1", y="method2", hue=mx.columns[0], data=flt_mx)
									if threshold != "":
										extension = "_thr" + str(threshold) + "_tendency.png"
									else:
										extension = "_tendency.png"
									plt.savefig(comp1 + extension)
									comparison = comp1
									plt.close()
	return comparison

def get_heatmap(filename,color,p1,p2,threshold):
	# HEATMAP
	mx = pandas.read_table(filename)
	mx = mx.drop(mx.columns[0], axis=1)
	comparison = str(p1) + "_vs_" + str(p2)
	if threshold != "":
		mx = mx.iloc[:, : int(threshold) + 1]
		mx = mx.head(int(threshold) + 1)
	mx = mx.set_axis(range(0,len(mx.columns)), axis=1)
	hm = sn.heatmap(data = mx, cmap = color)
	plt.xlabel(p2) 
	plt.ylabel(p1)
	hm.set(title="Congruence Score (" + str(p1) + " vs " + str(p2) + ")")
	#plt.savefig(comparison + "_final_score.png")
	#plt.show()
	fig = hm.get_figure()
	fig.set_size_inches(8, 6, forward=True)
	if threshold != "":
		fig.savefig(comparison + "_thr" + str(threshold) + "_final_score.png", format = "png", dpi=500)
	else:
		fig.savefig(comparison + "_final_score.png", format = "png", dpi=500)

def main():
	
	# argument options
    
	parser = argparse.ArgumentParser(prog="heatmap_final_score.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									################################################################################             
									#                                                                              #
									#                            heatmap_final_score.py                            #
									#                                                                              #
									################################################################################ 
									This script can be used to generate the heatmap of the congruence score obtained 
									for the comparison of two pipelines at all threshold levels, as well as 
									determine the tendency of the respective corresponding points.
									It takes as input the ALL_CORRESPONDENCE.tsv file (obtained with 
									get_best_part_correspondence.py) and the *final_score.tsv file obtained with 
									comparing_partitions_v2.py. 
									
									-------------------------------------------------------------------------------"""))
										
	parser.add_argument("--final-score", dest="final_score", required=False, type=str, help="[MANDATORY] *final_score.tsv output of comparing_partitions_v2.py with the CS matrix between two WGS pipelines.")
	parser.add_argument("--correspondence", dest="correspondence", required=True, type=str, help="[MANDATORY] Pivot table with the threshold correspondence (e.g. ALL_CORRESPONDENCE.tsv obtained with get_best_part_correspondence.py).")
	parser.add_argument("-t", "--threhold", dest="threshold", required=False, default="", help="[OPTIONAL] Maximum threshold to be used for plot")

	parser.add_argument("-c", "--color", dest="color", required=False, default="Blues", type=str, help="Seaborn color map. Default: Blues")
				
	args = parser.parse_args()
    
	filename = str(args.final_score).split("/")[-1]
	comparison_name = filename.split("_final_score.tsv")[0]
	pipeline1 = comparison_name.split("_vs_")[0]
	pipeline2 = comparison_name.split("_vs_")[1]

	get_tendency(args.correspondence, pipeline1, pipeline2, args.threshold)
	get_heatmap(filename, args.color, pipeline1, pipeline2, args.threshold)

if __name__ == "__main__":
    main()



#!/usr/bin/env	python3

"""
This script is used to identify the minimum partition required to cluster all samples within the same typing category (e.g. ST, serotype...)

By Veronica Mixao
@INSA
"""

import sys
import argparse
import textwrap
import pandas
import glob

version = "1.0.0"
last_updated = "2024-07-26"

# running the pipeline	----------

def main():
    
	# argument options
    
	parser = argparse.ArgumentParser(prog="poli_typing.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                                poli_typing.py                               #
									#                                                                             #
									###############################################################################  
									                            
									This script is used to identify the minimum partition required to cluster all 
									samples within the same typing category (e.g. ST, serotype...). It takes as
									input a directory with the *_metadata_w_partitions.tsv table generated by 
									ReporTree for one or multiple pipelines. This table should include a column with
									the typing category for which the analysis is required.
									
									-----------------------------------------------------------------------------"""))
	
	group0 = parser.add_argument_group("Input/Output", "Input/Output specifications")
	group0.add_argument("-d", "--dir", dest="directory", required=True, type=str, help="[MANDATORY] Directory with the *_metadata_w_partitions.tsv file generated by ReporTree.")
	group0.add_argument("-o", "--output", dest="out", required=True, type=str, help="[MANDATORY] Tag for output file name")
	
	group1 = parser.add_argument_group("Analysis", "Analysis specifications")
	group1.add_argument("-c", "--column", dest="column", required=False, type=str, default="MLST_ST", help="[OPTIONAL] Name of the column that should be used for analysis (default: MLST_ST).")
	
	args = parser.parse_args()

	variable_interest = args.column
	partition_code = ["MST-", "single-"]

	final = {}

	print("\n******************************\n")
	print("Running poli_typing.py")
	print(version, "last updated on", last_updated)

	for filename in glob.glob(args.directory + "/*_metadata_w_partitions.tsv"):
		print("Checking " + filename)
		columns_partitions = []
		mx = pandas.read_table(filename,dtype = str)
		if variable_interest not in mx.columns:
			sys.exit("ERROR!! The column name specified was not found in file " + filename)
		if variable_interest not in final.keys():
			final[variable_interest] = []
			final["size"] = []
			list_obs = mx[variable_interest].astype(str).values.tolist()
			val = set(list_obs)
			if 'nan' in val or 'NAN' in val or 'NaN' in val:
				val.remove('nan')
			for v in val:
				final[variable_interest].append(v)
				final["size"].append(list_obs.count(v))
		for col in mx.columns:
			for code in partition_code:
				if code in col:
					columns_partitions.append(col)					
		
		if "/" in filename:
			pip_code = filename.split("/")[-1]
		else:
			pip_code = filename
		pipeline = pip_code.split("_metadata_w_partitions.tsv")[0]
		if pipeline not in final.keys():
			final[pipeline] = []
		for variable in final[variable_interest]:
			check = False
			flt_mx = mx[(mx[variable_interest].astype(str) == variable)]
			for col in columns_partitions:
				clusters = set(flt_mx[col].astype(str).values.tolist())
				if 'nan' in clusters:
					clusters.remove('nan')
				n_clusters = len(clusters)
				if n_clusters == 1:
					check = True
					if "MST-" in col:
						col = col.split("MST-")[1]
					elif "single-" in col:
						col = col.split("single-")[1]
					if "x" in col:
						col = col.split("x")[0]
					final[pipeline].append(col)
					break
			if not check:
				final[pipeline].append("-")
				
	final_matrix = pandas.DataFrame(data = final)
	final_matrix.to_csv(args.out + "_poli.tsv", index = False, header=True, sep ="\t")

if __name__ == "__main__":
    main()


#!/usr/bin/env  python3

"""
This script generates all the plots used for the BeONE cluster congruence analysis

By Veronica Mixao
@INSA
"""

import argparse
import textwrap
import glob
import pandas
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import math

version = "1.0.0"
last_updated = "2024-07-26"

def get_n_clusters(directory):
    """ This function determines the number of clusters found in each partition for all partitions file in a directory 
    input: directory
    output: dataframe
    """
    
    info_partitions = {"pipeline": [], "threshold": [], "partitions": []}
    order_col = ["pipeline", "threshold", "partitions"]

    for filename in glob.glob(directory + "/*_partitions.tsv"):
        fname = filename.split("/")[-1]
        pipeline = fname.split("_partitions.tsv")[0]
        partitions = pandas.read_table(filename)
        for i in range(1,len(partitions.columns)):
            clusters = pandas.unique(partitions[partitions.columns[i]])
            info_partitions["pipeline"].append(pipeline)
            info_partitions["threshold"].append(i)
            info_partitions["partitions"].append(len(clusters))
    
    matrix = pandas.DataFrame(data = info_partitions, columns = order_col)

    return matrix

def get_log_stability(directory):
	""" This function transforms the stableRegion files into a single matrix with the region coordenated represented in log2 
	input: directory
	output: dataframe
	"""

	info_blocks = {"pipeline": [], "block": [], "pipeline_code": [], "partition": []}
	order_col = ["pipeline", "block", "pipeline_code", "partition"]
	order_blocks = []
	for filename in glob.glob(directory + "/*_stableRegions.tsv"):
		fname = filename.split("/")[-1]
		pipeline = fname.split("_stableRegions.tsv")[0]
		block_code = 1
		with open(filename) as infile:
			lines = infile.readlines()
			for line in lines:
				if "#" not in line:
					lin = line.split("\n")[0]
					l = lin.split("\t")
					block = l[0]
					pipeline_code = pipeline + "_" + str(block_code)
					start_info = l[1]
					start_info = start_info.split("-")[1]
					start_info = start_info.split("x")[0]
					end_info = l[2]
					end_info = end_info.split("-")[1]
					end_info = end_info.split("x")[0]

					if int(l[3]) >= 10:
						start_log2 = math.log2(int(start_info))
						end_log2 = math.log2(int(end_info))
						#start_log2 = int(start_info)
						#end_log2 = int(end_info)

						# line with start
						info_blocks["pipeline"].append(pipeline)
						info_blocks["block"].append(block)
						info_blocks["pipeline_code"].append(pipeline_code)
						info_blocks["partition"].append(start_log2)

						# line with "median"
						info_blocks["pipeline"].append(pipeline)
						info_blocks["block"].append(block)
						info_blocks["pipeline_code"].append(pipeline_code)
						info_blocks["partition"].append(end_log2)		

						# line with "median"
						info_blocks["pipeline"].append(pipeline)
						info_blocks["block"].append(block)
						info_blocks["pipeline_code"].append(pipeline_code)
						info_blocks["partition"].append(end_log2)
						block_code += 1
						order_blocks.append(pipeline_code)
	
	matrix = pandas.DataFrame(data = info_blocks, columns = order_col)
	
	return matrix, order_blocks

def get_ST_info(filename, col, col_n):
	""" This function counts the number of samples per ST and generates a matrix with this info
	input: directory
	output: dataframe
	"""

	counts = {col: [], col_n: []}

	info = pandas.read_table(filename, dtype={col:"object"})
	info[col].replace('', np.nan, inplace = True)
	info.dropna(subset=[col], inplace = True)
	ST_lst = info[col].values.tolist()
	ST = set(ST_lst)
	for st in ST:
		c = ST_lst.count(st)
		if c >= 50:
			counts[col].append(st)
			counts[col_n].append(c)
	
	matrix = pandas.DataFrame(data = counts)
	matrix = matrix.sort_values(by=[col_n], ascending=False)

	return matrix

def get_polityping_info(filename, min_length):
	""" This function transforms the polityping output
	input: directory
	output: dataframe
	"""

	info = {}

	i = 0
	with open(filename) as infile:
		lines = infile.readlines()
		for line in lines:
			lin = line.split("\n")[0]
			l = lin.split("\t")

			if i == 0:
				main_col = l[0]
				size_col = l[1]
				info[main_col] = []
				info[size_col] = []
				info["pipeline"] = []
				info["threshold"] = []
				pipelines = l[2:]
			else:
				if int(l[1]) >= min_length:
					for k in range(0,len(pipelines)):
						if l[k+2] != "-":
							info[main_col].append(l[0])
							info[size_col].append(int(l[1]))
							info["pipeline"].append(pipelines[k])
							info["threshold"].append(int(l[k+2]))
			i += 1

	matrix = pandas.DataFrame(data = info)
	matrix = matrix.sort_values(by=[size_col], ascending=False)

	return matrix, main_col, pipelines


def main():
		# argument options
    
	parser = argparse.ArgumentParser(prog="congruence_plots.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                             congruence_plots.py                             #
									#                                                                             #
									############################################################################### 
									This script generates useful plots for the WGS cluster congruence analysis.
																																				
									Required input files:
									- Plot Clusters_per_partition: directory with *_partitions.tsv from ReporTree
									- Plot Stability: directory with *_stableRegions.tsv from ReporTree
									- ST_barplot: *metadata_w_partitions.tsv file in "-f" and column name in "-col"
									- ST_swarmplot: *_poli.tsv file from poli_typing.py
									- ST_boxplot: *_poli.tsv from poli_typing.py
									-------------------------------------------------------------------------------"""))
										
	parser.add_argument("-d", "--directory", dest="directory", required=True, type=str, help="[MANDATORY] Directory with the input files.")
	parser.add_argument("-f", "--file", dest="filename", required=False, type=str, help="[ONLY REQUIRED FOR ST_barplot or ST_swarmplot or ST_boxplot] Filename.")
	parser.add_argument("-c", "--color", dest="color", required=False, default="Blues", type=str, help="Seaborn color map. Default: Blues")
	parser.add_argument("-t", "--tag", dest="tag", required=False, default="Congruence", type=str, help="Tag for output")
	parser.add_argument("-p", "--plot", dest="plot", required=False, default="Clusters_per_partition", type=str, help="Type of plot. Options: Clusters_per_partition, Stability, ST_barplot, ST_swarmplot, ST_boxplot")   
	parser.add_argument("-col", "--column", dest="column", required=False, type=str, help="[ONLY REQUIRED FOR ST_barplot] Column name (e.g. MLST_ST).")   

	args = parser.parse_args()

	print("\n******************************\n")
	print("Running beone_congruence_plots.py")
	print(version, "last updated on", last_updated)

	### get line plot with the number of clusters per partition
	if args.plot == "Clusters_per_partition":
		matrix = get_n_clusters(args.directory)
		matrix.to_csv(args.tag + "_clusters_partition.tsv", index = False, header=True, sep ="\t")
		lineplot = sns.lineplot(data=matrix,  x="threshold", y="partitions", hue="pipeline", palette="deep")

		fig = lineplot.get_figure()
		fig.savefig(args.tag + "_lineplot.svg", format = "svg")

	### get boxplot with the stability blocks
	if args.plot == "Stability":
		matrix, order_blocks = get_log_stability(args.directory)
		matrix = matrix.sort_values(by=["pipeline_code"])
		box_plot = sns.boxplot(data=matrix, x="partition", y="pipeline_code", order=order_blocks)
		fig = box_plot.get_figure()
		fig.savefig(args.tag + "_stability.svg", format = "svg") 
    
	### get ST barplot
	elif args.plot == "ST_barplot":
		col = args.column
		col_n = args.column + "_n"
		matrix = get_ST_info(args.directory + "/" + args.filename, col, col_n)
		order_ST = matrix[col].values.tolist()
		bar_plot = sns.barplot(data=matrix, x=col, y=col_n, order=order_ST)
        
		fig = bar_plot.get_figure()
		fig.savefig(args.tag + "_barplot.svg", format = "svg")
	
	### get ST swarmplot
	elif args.plot == "ST_swarmplot":
		matrix, main_col, order_pipelines = get_polityping_info(args.directory + "/" + args.filename, 50)
		matrix = matrix.sort_values(by=matrix.columns[1], ascending=False)
		#matrix = matrix.sort_values(by=["pipeline"], key=lambda col: col.str.lower())
		swarmplot = sns.swarmplot(data=matrix, x=main_col, y="threshold", hue="pipeline", hue_order = order_pipelines, palette="deep")
		fig = swarmplot.get_figure()
		fig.savefig(args.tag + "_swarmplot.svg", format = "svg")

    ### get ST boxplot
	elif args.plot == "ST_boxplot":
		matrix, main_col, order_pipelines = get_polityping_info(args.directory + "/" + args.filename, 2)
		box_plot = sns.boxplot(data=matrix, x="threshold", y="pipeline", order=order_pipelines)
        
		fig = box_plot.get_figure()
		fig.savefig(args.tag + "_boxplot.svg", format = "svg")

if __name__ == "__main__":
    main()


#!/usr/bin/env  python3

"""
This script was designed to compare the sample distances when a cluster zoom-in
is performed with a wgMLST schema. 

By Veronica Mixao
@INSA
"""

import os
import argparse
from doctest import ELLIPSIS_MARKER
import textwrap
import pandas
import glob

version = "1.0.0"
last_updated = "2024-07-26"

def check_dist(new_dist_file):
	""" Get info from the new allele distance matrix
	input: filename
	output: max_AD, samples  """

	mx = pandas.read_table(new_dist_file)
	samples = mx[mx.columns[0]].values.tolist()
	mx = mx.set_index(mx.columns[0], drop = True)
	max_value = mx.max().max()
	cluster_len = len(samples)
	if "SRR3973058" in samples:
		print(max_value)
	
	return max_value, samples, cluster_len

def check_old_dist(original_distances, samples):
	""" Get info from the old allele distance matrix
	input: filename
	output: max_AD """

	mx = pandas.read_table(original_distances)
	mx = mx.loc[mx[mx.columns[0]].isin(samples)]
	mx = mx[samples]
	max_value = mx.max().max()
	
	return max_value

def main():
    
	# argument options	----------
    
	parser = argparse.ArgumentParser(prog="wgmlst_exercise.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                              wgmlst_exercise.py                             #
									#                                                                             #
									############################################################################### 
									                            
									This script was designed to compare  the sample distances when a cluster zoom-in
									is performed with a wgMLST schema. 


									                  
									-------------------------------------------------------------------------------"""))
	
	## parameters
	
	parser.add_argument("-d", "--directory", dest="directory", required=True, type=str, help="PATH to directory with the different cluster zoom-in directories (generated by ReporTree)")
	parser.add_argument("-dist", "--original-distances", dest="original_distances", required=True, type=str, help="Distance matrix of the original analysis in which the clusters were determined")
	parser.add_argument("-o", "--output", dest="output", required=True, type=str, help="Tag for output filename")
	
	args = parser.parse_args()
	
	print("\n******************************\n")
	print("Running wgmlst_exercise.py")
	print(version, "last updated on", last_updated)

	## pipeline
	with open(args.output + "_dist_wgMLST.tsv", "w+") as out:
		print("cluster\tcluster_len\tdist_all\tdist_zoom\tdifference\tsamples", file = out)
		for dir in glob.glob(args.directory + "/*"):
			if "cluster_" in dir:
				new_dist_file = dir + "/cluster_dist_hamming.tsv"
				if os.path.exists(new_dist_file):
					max_value, samples, cluster_len = check_dist(new_dist_file)
					old_max = check_old_dist(args.original_distances, samples)
					dif = int(max_value) - int(old_max)
					if "/" in dir:
						cluster = dir.split("/")[-1]
					else:
						cluster = dir
					print("\t", cluster)
					print(str(cluster) + "\t" + str(cluster_len) + "\t" + str(old_max) + "\t" + str(max_value) + "\t" + str(dif) + "\t" + ",".join(samples), file = out)

if __name__ == "__main__":
    main()


#!/usr/bin/env	python3

"""
This script was designed to determine the partition correspondence between two pipelines.
It takes as input the indication of a directory with all the *final_score.tsv files (obtained with comparing_partitions_v2.py). 

By Veronica Mixao
@INSA
"""

import sys
import argparse
import textwrap
import glob
import pandas

version = "1.0.1"
last_updated = "2025-01-09"

def get_correspondence_all(input_dir, score):
    final = {"comparison": [], "method1": [], "method2": []}
    print("\n" + "Checking:")

    for filename in glob.glob(input_dir + "/*final_score.tsv"):
        with open(filename) as infile:
            counter = 0
            f_lines = infile.readlines()
            if "/" in filename:
                fname = filename.split("/")[-1]
            else:
                fname = filename
            comparison = fname.split("_final_score.tsv")[0]
            print("\t" + comparison)
            for line in f_lines:
                lin = line.split("\n")
                l = lin[0].split("\t")
                if counter > 0:
                    max_value = max(l[1:])
                    pos = [i for i, j in enumerate(l[1:]) if j == max_value]
                    if len(pos) == 1:
                        method1_pos = counter - 1
                        method2_pos = pos[0]
                    else:
                        method1_pos = counter - 1
                        dists = []
                        for val in pos:
                            dist = abs(counter - int(val))
                            dists.append(dist)
                        min_dist = min(dists)
                        pos2 = [i for i, j in enumerate(dists) if j == min_dist]
                        method2_pos = pos[pos2[0]]
                    if float(max_value) >= score:
                        final["comparison"].append(comparison)
                        final["method1"].append(method1_pos)
                        final["method2"].append(method2_pos)
                    else:
                        final["comparison"].append(comparison)
                        final["method1"].append(method1_pos)
                        final["method2"].append("-")
                counter += 1    
        mx = pandas.read_table(filename)
        counter2 = 0
        if "/" in filename:
            fname = filename.split("/")[-1]
        else:
            fname = filename
        comparison = fname.split("_final_score.tsv")[0] + "_rev"
        for info in mx.columns[1:]:
            max_value = max(mx[info])
            pos = [i for i, j in enumerate(mx[info]) if j == max_value]
            if len(pos) == 1:
                method1_pos = counter2
                method2_pos	= pos[0]
            else:
                method1_pos	= counter2
                dists = []
                for val in pos:
                    dist = abs(counter2 - int(val))
                    dists.append(dist)
                min_dist = min(dists)
                pos2 = [i for i, j in enumerate(dists) if j == min_dist]
                method2_pos = pos[pos2[0]]
            if float(max_value) >=	score:
                final["comparison"].append(comparison)
                final["method1"].append(method1_pos)
                final["method2"].append(method2_pos) 
            else:
                final["comparison"].append(comparison) 
                final["method1"].append(method1_pos) 
                final["method2"].append("-")
            counter2 += 1
            
    final_results = pandas.DataFrame(data = final)
    final_results.to_csv(input_dir + "/ALL_CORRESPONDENCE.tsv", index = False, header=True, sep = "\t")

def main():
	# argument options
    
    parser = argparse.ArgumentParser(prog="get_best_part_correspondence_threshold.py ", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                  get_best_part_correspondence_threshold.py                  #
									#                                                                             #
									############################################################################### 
									
									This script was designed to determine the partition correspondence between two 
									pipelines. It takes as input the indication of a directory with all the 
									*final_score.tsv files (obtained with comparing_partitions_vs.py) for which 
									the partition correspondence must be determined. In the end, it generates the
									ALL_CORRESPONDENCE.tsv file with the best partition correspondence for each
									pairwise pipeline comparison. A threshold of 2.85 is the default minimum score
									to determine this correspondence. It can be modified with "--score".
                                                                                                                                                   
									-------------------------------------------------------------------------------"""))
										
    parser.add_argument("-i", "--inputdir", dest="inputdir", required=True, type=str, help="[MANDATORY] Input directory with all the *final_score.tsv files from the all-to-all partition comparison between two pipelines (obtained with comparing_partitions_v2.py).")
    parser.add_argument("-s", "--score", dest="score", required=False, default=2.85, type=float, help="[OPTIONAL] Minimum score to consider two partitions as a correspondence. Default: 2.85.")

    args = parser.parse_args()
	
    print("\n******************************\n")
    print("Running get_best_part_correspondence_threshold.py")
    print(version, "last updated on", last_updated)

    get_correspondence_all(args.inputdir, args.score)

if __name__ == "__main__":
    main()



#!/usr/bin/env	python3

"""
This script was designed to determine pipeline comparability at outbreak-level. 
It takes as input the cluster composition file of each pipeline and reports, 
for each cluster at a given level, what is the level at which the same cluster 
is detected in another pipeline.

By Veronica Mixao
@INSA
"""

import sys
import argparse
import textwrap
import pandas

version = "1.0.0"
last_updated = "2024-07-26"

def get_pipelines(infile):
	""" Get the list of pipelines to compare
	input: infile with the path to the cluster composition files
	output: dictionary """

	pipelines = {}
	types = {}

	with open(infile) as path:
		lines = path.readlines()
		for line in lines:
			f = line.split("\n")[0]
			f = f.split("\t")
			if "/" in f[0]:
				fname = f[0].split("/")[-1]
				pipeline_name = fname.split("_clusterComposition")[0]
			else:
				pipeline_name = f[0].split("_clusterComposition")[0]
			print("\t" + pipeline_name)
			pipelines[pipeline_name] = f[0]
			types[pipeline_name] = f[1]
	
	return pipelines, types

def get_samples_exclude(pipelines, types):
	""" Determine samples to exclude from the analysis, i.e. absent from at least one pipeline
	input: dictionary
	output: set """

	info = {}
	all_samples = set()
	exclude_samples = set()

	for pipeline in pipelines.keys():
		if types[pipeline] == "allele":
			with open(pipelines[pipeline]) as cluster_composition_file:
				lines = cluster_composition_file.readlines()
				line_interest = lines[-1]
				lin = line_interest.split("\n")[0]
				l = lin.split("\t")
				cluster = l[3]
				info[pipeline] = set()
				for sample in cluster.split(","):
					info[pipeline].add(sample)
					all_samples.add(sample)

	for pipeline in info.keys():
		pipeline_samples = info[pipeline]
		for sample in all_samples:
			if sample not in pipeline_samples:
				exclude_samples.add(sample)

	return exclude_samples

def get_clusters(pipelines,exclude_samples):
	""" Identify the clusters which will be used for comparison
	input: dictionary with the files to retrieve clusters from
	output: dictionary """

	clusters = {} # clusters[pipeline][clusters] = partition
	partitions = {} # partitions[pipeline][partition] = [clusters]

	for pipeline in pipelines.keys():
		clusters[pipeline] = {}
		partitions[pipeline] = {}
		with open(pipelines[pipeline]) as cluster_composition_file:
			lines = cluster_composition_file.readlines()
			for line in lines[1:]:
				lin = line.split("\n")[0]
				l = lin.split("\t")
				partition_name = l[0]
				cluster = l[3]
				if "single" in partition_name:
					partition = partition_name.split("single-")[1]
					partition = partition.split("x")[0]
				elif "MST" in partition_name:
					partition = partition_name.split("MST-")[1]
					partition = partition.split("x")[0]
				else:
					print("Unknown method in partition name of pipeline " + str(pipeline))
					sys.exit(1)
				if int(partition) not in partitions[pipeline].keys():
					partitions[pipeline][int(partition)] = []
				
				samples = cluster.split(",")
				cluster = list(set(samples) - exclude_samples)
				cluster.sort()
				
				if len(cluster) > 1:
					cluster = ",".join(cluster)
					partitions[pipeline][int(partition)].append(cluster)
					if cluster not in clusters[pipeline].keys():
						clusters[pipeline][cluster] = partition

	return clusters, partitions

def get_clusters_of_interest(partitions,threshold,types):
	""" Get the set of clusters of interest
	input: partitions dictionary and the threshold where to search
	output: set """

	clusters_of_interest = set()
	for pipeline in partitions.keys():
		if types[pipeline] == "allele":
			for cluster in partitions[pipeline][int(threshold)]:
				clusters_of_interest.add(cluster)
	
	return clusters_of_interest

def get_matrix(clusters_of_interest,clusters,types):
	""" generate the final matrix
	input: clusters of interest and the clusters dictionary
	output: matrix """
	
	mx = {}
	mx["cluster"] = []
	mx["cluster_length"] = []
	mx["n_allele_pipelines"] = []
	pipeline_lst = []

	for cluster in clusters_of_interest:
		mx["cluster"].append(cluster)
		cluster_counter = 0
		for pipeline in clusters.keys():
			if pipeline not in mx.keys():
				mx[pipeline] = []
				pipeline_lst.append(pipeline)
			if cluster in clusters[pipeline]:
				mx[pipeline].append(clusters[pipeline][cluster])
				if types[pipeline] == "allele":
					cluster_counter += 1
			else:
				mx[pipeline].append("-")
		mx["cluster_length"].append(len(cluster.split(",")))
		mx["n_allele_pipelines"].append(str(cluster_counter))
	
	return mx, pipeline_lst

def get_pivot(mx, pipeline_lst):
	""" generate the pivot of the final matrix
	input: dictionary
	output: matrix 
	"""

	pivot_mx = {"n_allele_pipelines": [], "cluster_length": [], "pipeline": [], "threshold": []}

	for i in range(0,len(mx["cluster_length"])):
		for pipeline in pipeline_lst:
			if mx[pipeline][i] != "-":
				pivot_mx["n_allele_pipelines"].append(mx["n_allele_pipelines"][i])
				pivot_mx["cluster_length"].append(mx["cluster_length"][i])
				pivot_mx["pipeline"].append(pipeline)
				pivot_mx["threshold"].append(mx[pipeline][i])
	
	pivot_mx_df = pandas.DataFrame.from_dict(pivot_mx)

	return pivot_mx_df

def main():
    
	# argument options	----------
    
	parser = argparse.ArgumentParser(prog="comparison_outbreak_level.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                        comparison_outbreak_level.py                         #
									#                                                                             #
									############################################################################### 
									                            
									This script was designed to determine pipeline comparability at outbreak-level. 
									It takes as input the cluster composition file of each pipeline and reports, 
									for each cluster at a given level, what is the level at which the same cluster 
									is detected in another pipeline.

									Briefly:
									1. Create a list of genetic clusters
									1.1 Pipeline 1: identify all clusters at threshold X
									1.2 Pipeline 2: identify all clusters at threshold X
									1.3 ...

									2. For each genetic cluster, determine the threshold at which it is identified
									by each pipeline

									3. Present the results in a tsv matrix

									                  
									-------------------------------------------------------------------------------"""))
	
	## parameters
	
	parser.add_argument("-i", "--input", dest="infile", required=True, type=str, help="Input *tsv file containing the path to each *clusterComposition.tsv file that has to be compared and the type of pipeline, i.e. allele or snp (cluster composition files can be obtained with ReporTree)")
	parser.add_argument("-o", "--out", dest="output", required=True, type=str, help="Tag for output file")
	parser.add_argument("-t", "--threshold", dest="threshold", required=True, type=int, help="Threshold at which the genetic clusters must be identified")
	
	args = parser.parse_args()
	
	print("\n******************************\n")
	print("Running comparison_outbreak_level.py")
	print(version, "last updated on", last_updated)

	## pipeline
	
	print("\nDetecting pipelines...")
	pipelines, types = get_pipelines(args.infile)

	print("Identifying samples that were absent from at least one allele pipeline...")
	exclude_samples = get_samples_exclude(pipelines, types)

	print("Getting cluster information...")
	clusters, partitions = get_clusters(pipelines,exclude_samples)

	print("Determining clusters of interest...")
	clusters_of_interest = get_clusters_of_interest(partitions,args.threshold,types)

	print("Pipeline comparison...")
	mx, pipeline_lst = get_matrix(clusters_of_interest,clusters,types)
	mx_df = pandas.DataFrame.from_dict(mx)

	print("Generating pivot table...")
	mx_pivot = get_pivot(mx, pipeline_lst)

	mx_df.to_csv(args.output + ".tsv", index = False, header=True, sep ="\t")
	mx_pivot.to_csv(args.output + "_pivot.tsv", index = False, header=True, sep ="\t")

if __name__ == "__main__":
    main()


import sys
import math
import pandas 
import numpy as np
# Original Metrics code by Peter Kruczkiewicz (https://github.com/peterk87)
#National Microbiology Laboratory at Lethbridge, Public Health Agency of Canada, Lethbridge, Alberta, Canada. 
# https://gist.github.com/jacarrico/82c32d05e90f43dec2b446cdb1ed46c6
def getContTable(ar1, ar2):
    cont = {}
    for i in xrange(0, len(ar1)):
        keyAr1 = ar1[i]
        keyAr2 = ar2[i]
        if keyAr1 in cont:
            if keyAr2 in cont[keyAr1]:
                cont[keyAr1][keyAr2] += 1
            else:
                cont[keyAr1][keyAr2] = 1
        else:
            cont[keyAr1] = {keyAr2: 1}
    return cont


def getContTableTotals(cont, ar1, ar2):
    sumRow = {}
    sumCol = {}
    h1 = set(ar1)
    h2 = set(ar2)
    for x in h2:
        sumRow[x] = 0
        for y in h1:
            if y in cont:
                if x in cont[y]:
                    val = cont[y][x]
                    sumRow[x] += val
                    if y in sumCol:
                        sumCol[y] += val
                    else:
                        sumCol[y] = val
    total = 0
    for x in h1:
        total += sumCol[x]
    return (sumRow, sumCol, total)


def getMismatchMatrix(cont, ar1, ar2):
    totals = getContTableTotals(cont, ar1, ar2)
    # print totals
    n = totals[2]
    h1 = set(ar1)
    h2 = set(ar2)
    a = 0
    for x in h1:
        for y in h2:
            if x in cont:
                if y in cont[x]:
                    val = cont[x][y]
                    a += (val * (val - 1)) / 2
    a1 = 0
    sumCol = totals[1]
    for x in sumCol:
        val = sumCol[x]
        a1 += (val * (val - 1)) / 2

    b = a1 - a

    a2 = 0
    sumRow = totals[0]
    for x in sumRow:
        val = sumRow[x]
        a2 += (val * (val - 1)) / 2
    c = a2 - a
    d = float((n * (n - 1)) / 2) - a1 - c
    return (a, b, c, d, n)


def getRand(a, b, c, d):
    rand = (a + d) / float(a + b + c + d)
    return rand


def getWallace(a, b, c):
    w1 = float(0)
    w2 = float(0)
    if (a + b) > 0:
        w1 = a / float(a + b)
    if (a + c) > 0:
        w2 = a / float(a + c)
    return (w1, w2)


def getSimpsons(ar):
    n = len(ar)
    d = {}
    for x in ar:
        if x in d:
            d[x] += 1
        else:
            d[x] = 1

    sumTotal = 0
    sumFc2 = 0
    sumFc3 = 0
    for x in d:
        val = d[x]
        sumTotal += val * (val - 1)
        sumFc2 += (val / float(n)) ** 2.0
        sumFc3 += (val / float(n)) ** 3.0

    sid = 1.0
    if n * (n - 1) > 0:
        sid = 1.0 - (sumTotal / float(n * (n - 1)))

    sqSumFc2 = sumFc2 ** 2.0
    s2 = (4.0 / float(n)) * float(sumFc3 - sqSumFc2)

    sidLow = sid - 2 * math.sqrt(s2)
    sidHigh = sid + 2 * math.sqrt(s2)

    return (sid, sidLow, sidHigh, len(d))


def getAdjustedWallace(cont, ar1, ar2):
    sumCol, sumRow, totals = getContTableTotals(cont, ar1, ar2)
    rSumW1 = 0
    rSumW2 = 0
    # For Rows
    for i in cont:
        rSum = sumRow[i]
        rsumFc2 = 0
        rsumFc3 = 0
        for j in cont[i]:
            val = cont[i][j]
            #print 'i',i,'j',j,'val',val
            cSum = sumCol[j]
            rsumFc2 += (val / float(rSum)) ** 2.0
            rsumFc3 += (val / float(rSum)) ** 3.0

        rsqrsumFc2 = rsumFc2 ** 2.0
        rvarSID = 0.0
        if rSum > 1:
            rvarSID = float(4.0 * rSum * (rSum - 1.0) * (rSum - 2.0) * rsumFc3 + 2.0 * rSum * (rSum - 1.0) * rsumFc2 - 2.0 * rSum * (rSum - 1.0) * (2.0 * rSum - 3.0) * rsqrsumFc2) / float((rSum * (rSum - 1.0)) ** 2.0)
        rSumW1 += float((rSum * (rSum - 1.0)) ** 2.0) * rvarSID
        rSumW2 += rSum * (rSum - 1.0)

    varW1 = 0.0
    if rSumW2 > 0:
        varW1 = float(rSumW1 / float(float(rSumW2) ** 2.0))

    csumw1 = 0
    csumW2 = 0
  

    a, b, c, d, n = getMismatchMatrix(cont, ar1, ar2)
    w1, w2 = getWallace(a, b, c)
    
    sid1 = getSimpsons(ar1)
    sid2 = getSimpsons(ar2)
    
    wi1 = 1 - sid1[0]
    wi2 = 1 - sid2[0]
    
    aw1 = float(w1 - wi2) / float(1 - wi2)
    
    aw1CI = 2.0 * (1.0 / float(1.0 - wi2)) * math.sqrt(varW1)
    aw1Low = aw1 - aw1CI
    aw1High = aw1 + aw1CI
    return (aw1, aw1Low, aw1High)


print len(sys.argv)  
if len(sys.argv) == 2:
    filename  = sys.argv[1]
    data = pandas.read_table(filename)
    for col1 in data.columns:
        for col2 in data.columns:
            if col1 != col2:
                print "===========",col1,"=",col2,"============"
                icol = data[col1].values.tolist()
                jcol = data[col2].values.tolist()
                cont = getContTable(icol, jcol)
                cont_2 = getContTable(jcol, icol)
                abcdn = getMismatchMatrix(cont, icol, jcol)
                abcdn_2 = getMismatchMatrix(cont_2, jcol, icol)
                #print "Rand\t%f" % getRand(abcdn[0], abcdn[1], abcdn[2], abcdn[3])

                sid1 = getSimpsons(icol)
                sid2 = getSimpsons(jcol)      
                print "Simpsons ",col1,"\t%f\t%f\t%f\t%d" % sid1
                print "Simpsons ",col2,"\t%f\t%f\t%f\t%d" % sid2
                print "\n"
                wallace = getWallace(abcdn[0], abcdn[1], abcdn[2])
                print "Wallace ",col1,"vs ",col2,"\t%f" % wallace[0]
                print "Wallace ",col2,"vs",col1,"\t%f" % wallace[1]
                print "\n"
                AdjustedWallace = getAdjustedWallace(cont, icol, jcol)
                AdjustedWallace_2 = getAdjustedWallace(cont_2, jcol, icol)

                print "Adjusted Wallace ",col1,"vs ",col2,"\t%f\t%f\t%f" % AdjustedWallace
                print "Adjusted Wallace ",col2,"vs",col1,"\t%f\t%f\t%f" % AdjustedWallace_2 
                print "\n"




#!/usr/bin/env  python3

"""
This script was designed to assess the percentage of clusters detected by one pipeline that are also detected, 
with the exact same composition, by another pipeline.

By Veronica Mixao
@INSA
"""

import sys
import argparse
import textwrap
import pandas

version = "1.0.0"
last_updated = "2024-07-26"

def main():
    
	# argument options	----------
    
    parser = argparse.ArgumentParser(prog="stats_outbreak_analysis.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                          stats_outbreak_analysis.py                         #
									#                                                                             #
									############################################################################### 
									                            
									This script was designed to assess the percentage of clusters detected by one 
									pipeline that are also detected, with the exact same composition, by another 
									pipeline.
									                  
									-------------------------------------------------------------------------------"""))
	
    ## parameters
	
    parser.add_argument("-i", "--input", dest="infile", required=True, type=str, help="Input *tsv file containing the path to each *clusterComposition.tsv file that has to be compared (these files \
                        can be obtained with ReporTree)")
    parser.add_argument("-o", "--out", dest="output", required=True, type=str, help="Tag for output file")
    parser.add_argument("-c", "--comparison", dest="comparison", required=True, type=str, help="Type of comparison ('equal' for the assessment of whether a cluster is detected at a given threshold by \
                        another pipeline; 'lower_equal' for the assessment of wether a cluster is detected at up to a given threshold by another pipeline)")
    parser.add_argument("-t1", "--threshold1", dest="threshold1", required=True, type=int, help="Threshold at which the genetic clusters must be identified for the pipeline of interest")
    parser.add_argument("-t2", "--threshold2", dest="threshold2", required=True, type=int, help="Threshold at which the genetic clusters must be searched in the other pipelines")
	
    args = parser.parse_args()
	
    print("\n******************************\n")
    print("Running stats_outbreak_analysis.py")
    print(version, "last updated on", last_updated)

    clusters = {}
    clusters7 = {}
    samples = {}

    files = []
    thr_min = args.threshold1
    thr_max = args.threshold2
    comp_type = args.comparison # options: lower_equal or equal
    input_file = args.infile
    tag_output = args.output

    with open(input_file) as infile:
        lines = infile.readlines()
        for line in lines:
            l = line.split("\n")[0]
            files.append(l)

    for filename in files:
        if "/" in filename:
            fname = filename.split("/")[-1]
            pipeline_name = fname.split("_clusterComposition")[0]
        else:
            pipeline_name = filename.split("_clusterComposition")[0]
        
        info = {}
        all_samples = set()
        with open(filename) as cluster_composition_file:
            lines = cluster_composition_file.readlines()

            # determine all samples used in the analysis
            line_interest = lines[-1]
            lin = line_interest.split("\n")[0]
            l = lin.split("\t")
            cluster = l[3]
            samples_pipeline = set()
            for sample in cluster.split(","):
                samples_pipeline.add(sample)
            samples[pipeline_name] = samples_pipeline

            # get clusters at 7
            for line in lines[1:]:
                lin = line.split("\n")[0]
                l = lin.split("\t")
                partition_name = l[0]
                cluster = l[3]
                cluster_len = int(l[2])
                if "single" in partition_name:
                    partition = partition_name.split("single-")[1]
                    partition = partition.split("x")[0]
                elif "MST" in partition_name:
                    partition = partition_name.split("MST-")[1]
                    partition = partition.split("x")[0]
                else:
                    print("Unknown method in partition name of pipeline " + str(pipeline_name))
                    sys.exit(1)

                if comp_type == "lower_equal":    
                    if int(partition) <= thr_max and cluster_len >= 2:
                        if pipeline_name not in clusters.keys():
                            clusters[pipeline_name] = set()
                        clusters[pipeline_name].add(cluster)
                    if int(partition) == thr_min and cluster_len >= 2:
                        if pipeline_name not in clusters7.keys():
                            clusters7[pipeline_name] = set()
                        clusters7[pipeline_name].add(cluster)
                elif comp_type == "equal":
                    if int(partition) == thr_max and cluster_len >= 2:
                        if pipeline_name not in clusters.keys():
                            clusters[pipeline_name] = set()
                        clusters[pipeline_name].add(cluster)
                    if int(partition) == thr_min and cluster_len >= 2:
                        if pipeline_name not in clusters7.keys():
                            clusters7[pipeline_name] = set()
                        clusters7[pipeline_name].add(cluster)
                else:
                    print("I do not know what to do... no valid comparison type provided!")
                    sys.exit()

    # pairwise comparisons
                            
    pairwise_comparison = {"pipeline": []}
    pairwise_comparison_pct = {"pipeline": []}
    n_pipelines = len(clusters.keys())
    pipeline_info = {}
    col_order = ["pipeline"]
    cluster_info = {}

    with open(tag_output + "_missing_clusters_" + str(thr_min) + "_" + str(comp_type) + "_" + str(thr_max) + ".tsv", "w+") as out_fail:
        for pipeline1 in clusters7.keys():
            cluster_info[pipeline1] = {}
            pairwise_comparison["pipeline"].append(pipeline1)
            pairwise_comparison_pct["pipeline"].append(pipeline1)
            col_order.append(pipeline1)
            clusters_pipeline1 = clusters7[pipeline1]
            total_clusters = len(clusters_pipeline1)
            shared_at_least_one = set()

            for pipeline2 in clusters.keys():
                shared_by_two = 0
                if pipeline2 not in pairwise_comparison.keys():
                    pairwise_comparison[pipeline2] = []
                    pairwise_comparison_pct[pipeline2] = []
                clusters_pipeline2 = clusters[pipeline2]

                for cluster1 in clusters_pipeline1:
                    found = False
                    if cluster1 not in cluster_info[pipeline1].keys():
                        cluster_info[pipeline1][cluster1] = 0
                    cluster1_composition = set(cluster1.split(",")) & samples[pipeline2]
                    for cluster2 in clusters_pipeline2:
                        cluster2_composition = set(cluster2.split(",")) & samples[pipeline1]
                        if len(cluster1_composition - cluster2_composition) == 0 and len(cluster2_composition - cluster1_composition) == 0: # same cluster
                            if len(cluster1_composition) > 1:
                                shared_by_two += 1
                                cluster_info[pipeline1][cluster1] += 1
                                found = True
                                continue
                    if not found:
                        print(pipeline1, pipeline2, cluster1, file = out_fail)
                            
                pairwise_comparison[pipeline2].append(shared_by_two)
                shared_by_two_pct = float(int(shared_by_two) / int(total_clusters))
                pairwise_comparison_pct[pipeline2].append(shared_by_two_pct)
            pipeline_info[pipeline1] = total_clusters

    pairwise_df = pandas.DataFrame(data = pairwise_comparison, columns = col_order)
    pairwise_pct_df = pandas.DataFrame(data = pairwise_comparison_pct, columns = col_order)
    pairwise_df.to_csv(tag_output + "_pairwise_comparison_" + str(thr_min) + "_" + str(comp_type) + "_" + str(thr_max) + ".tsv", index = False, header=True, sep ="\t")
    pairwise_pct_df.to_csv(tag_output + "_pairwise_comparison_" + str(thr_min) + "_" + str(comp_type) + "_" + str(thr_max) + "_pct.tsv", index = False, header=True, sep ="\t")

    with open(tag_output + "_summary_" + str(thr_min) + "_" + str(comp_type) + "_" + str(thr_max) + ".tsv", "w+") as out:
        header = ["pipeline\ttotal_clusters\tn\texact_composition_all\tshared\texclusive"]
        print("\t".join(header), file = out)
        for pipeline in pipeline_info.keys():
            total_clusters = pipeline_info[pipeline]
            n = 0
            exact = 0
            shared = 0
            exclusive = 0
            for cluster in cluster_info[pipeline].keys():
                n += 1
                n_pipelines_cluster = cluster_info[pipeline][cluster]
                if n_pipelines_cluster == len(pipeline_info.keys()): # all
                    exact += 1
                elif n_pipelines_cluster < len(pipeline_info.keys()) and n_pipelines_cluster > 1: # shared by at least one
                    shared += 1
                else:
                    exclusive += 1
            info = [pipeline, str(total_clusters), str(n), str(exact), str(shared), str(exclusive)]
            print("\t".join(info), file = out)

if __name__ == "__main__":
    main()


#!/usr/bin/env  python3

"""
This script sumarizes the distribution of a variable within the clusters at a given threshold

By Veronica Mixao
@INSA
"""

import argparse
import textwrap
import pandas

version = "1.0.0"
last_updated = "2024-07-26"

def get_info_col_summary(fname, mx, col, thr):
    """ Get stats per column and cluster """
    
    v_lst = mx[col].values.tolist()
    v = set(v_lst)
    N = 0
    N_single_cluster = 0
    N_single_cluster_100pct = 0
    N_split_2 = 0
    N_split_3_more = 0
    clusters2check = set()
    for val in v:
        if v_lst.count(val) > 1:
            N += 1
            flt_mx = mx[mx[col] == val]
            clusters_lst = flt_mx[thr].values.tolist()
            clusters = set(clusters_lst)
            if len(clusters) == 1: # all grouped into 1 cluster
                N_single_cluster += 1
                clusters2check.add(clusters_lst[0])
            elif len(clusters) == 2:
                N_split_2 += 1
            else:
                N_split_3_more += 1
    
    for cluster in clusters2check:
        flt_mx = mx[mx[thr] == cluster]
        sts_lst = flt_mx[col].values.tolist()
        sts = set(sts_lst)
        if len(sts) == 1:
            N_single_cluster_100pct += 1
    
    print(fname + "\t" + str(col) + "\t" + str(N) + "\t" + str(N_single_cluster) + "\t" + str(N_single_cluster_100pct) + "\t" + str(N_split_2) + "\t" + str(N_split_3_more))

def get_info_col(fname, mx, col, thr):
    with open(fname + "_" + col + "_" + thr + ".tsv", "w+") as out:
        print("fname\tcluster\tcluster_size\tmost_represented\tN_most_represented_dataset\tcounter_most_represented_cluster\tn_clusters_most_represented\tpct_most_represented_cluster\tpct_most_represented", file = out)
        v_lst = mx[col].values.tolist()
        v = set(v_lst)
        N = 0
        N_single_cluster = 0
        N_multiple_clusters = 0
        st_cluster_counter = {}
        for val in v:
            if v_lst.count(val) > 0:
                N += 1
                flt_mx = mx[mx[col] == val]
                clusters_lst = flt_mx[thr].values.tolist()
                clusters = set(clusters_lst)
                if len(clusters) == 1: # all grouped into 1 cluster
                    N_single_cluster += 1
                else:
                    N_multiple_clusters += 1
                st_cluster_counter[val] = len(clusters)
        
        all_clusters_lst = mx[thr].values.tolist()
        all_clusters = set(all_clusters_lst)
        N_clusters = 0
        N_single_info = 0
        N_multiple_info = 0
        for cluster in all_clusters:
            cluster_size = all_clusters_lst.count(cluster)
            if cluster_size > 0:
                N_clusters += 1
                flt_mx = mx[mx[thr] == cluster]
                info_lst = flt_mx[col].values.tolist()
                info = set(info_lst)
                if len(info) == 1:
                    N_single_info += 1
                    most_represented = info_lst[0]
                    counter = cluster_size
                else:
                    N_multiple_info += 1
                    counter = 0
                    most_represented = ""
                    for info_val in info:
                        counter_new = info_lst.count(info_val)
                        if counter_new > counter:
                            counter = counter_new
                            most_represented = info_val
                N_most_represented = len(mx[mx[col] == most_represented].values.tolist())
                if N_most_represented > 0:
                    pct_most_represented_cluster = int(counter)/float(cluster_size)
                    pct_most_represented = int(counter)/float(N_most_represented)
                    print(str(fname) + "\t" + str(cluster) + "\t" + str(cluster_size) + "\t" + str(most_represented) + "\t" + str(N_most_represented) + "\t" + str(counter) + "\t" + str(st_cluster_counter[most_represented]) + "\t" + str(pct_most_represented_cluster) + "\t" + str(pct_most_represented), file = out)

def main():
		# argument options
    
    parser = argparse.ArgumentParser(prog="get_stats_threshold.py", formatter_class=argparse.RawDescriptionHelpFormatter, description=textwrap.dedent("""\
									###############################################################################             
									#                                                                             #
									#                             get_stats_threshold.py                          #
									#                                                                             #
									############################################################################### 
									
									This script sumarizes the cluster distribution of a variable (e.g. MLST) within 
									the clustering results obtained with ReporTree at a given threshold.
                                                                                                                                                   
									-------------------------------------------------------------------------------"""))
										
    parser.add_argument("-m", "--metadata", dest="metadata", required=True, type=str, help="[MANDATORY] Metadata with partitions file (can be obtained with ReporTree).")
    parser.add_argument("-t", "--threshold", dest="threshold", required=True, help="Column name with the threshold used for reporting (e.g., single-150x1.0).")
    parser.add_argument("-col", "--column", dest="column", required=False, type=str, help="Column name with the variable to summarize (e.g. MLST_ST).")
    parser.add_argument("-a", "--analysis", dest="analysis", required=False, type=str, help="Type of analysis (summary or detail). Default: summary")

    args = parser.parse_args()
	
    print("\n******************************\n")
    print("Running get_stats_threshold.py")
    print(version, "last updated on", last_updated)

    mx = pandas.read_table(args.metadata)
    if "/" in args.metadata:
        name_info = args.metadata.split("/")[-1]
    else:
        name_info = args.metadata
    name = name_info.split(".")[0]
    col = args.column
    
    if args.analysis == "summary":
        get_info_col_summary(name, mx, col, args.threshold)
    elif args.analysis == "detail":
        get_info_col(name, mx, col, args.threshold)

if __name__ == "__main__":
    main()